"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pingMillis = exports.packInstructions = exports.airdrop = exports.getSolanaKeypair = exports.humanReadable = exports.humanReadableBigint = exports.roundToDecimals = exports.numberWithCommas = exports.supplyAsNum = exports.toBN = exports.amountAsNum = exports.toNumber = exports.truthy = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const web3_js_1 = require("@solana/web3.js");
const transaction_1 = require("./transaction");
const truthy = (value) => !!value;
exports.truthy = truthy;
function toNumber(numberOrBn, mintOrDecimals) {
    if (bn_js_1.default.isBN(numberOrBn) || typeof numberOrBn == "bigint") {
        return amountAsNum(numberOrBn, mintOrDecimals);
    }
    else {
        return numberOrBn;
    }
}
exports.toNumber = toNumber;
function amountAsNum(amount, mintOrDecimals) {
    const decimals = typeof mintOrDecimals === "number"
        ? mintOrDecimals
        : mintOrDecimals.decimals;
    const factor = new bn_js_1.default(Math.pow(10, decimals).toString());
    const decimal = new bn_js_1.default(amount.toString()).mod(factor).toNumber() / factor.toNumber();
    return new bn_js_1.default(amount.toString()).div(factor).toNumber() + decimal;
}
exports.amountAsNum = amountAsNum;
function toBN(numberOrBn, mintOrDecimals) {
    const decimals = typeof mintOrDecimals === "number"
        ? mintOrDecimals
        : mintOrDecimals.decimals;
    if (bn_js_1.default.isBN(numberOrBn)) {
        return numberOrBn;
    }
    else {
        return new bn_js_1.default(Number(Math.floor(Number(numberOrBn))))
            .mul(new bn_js_1.default(Math.pow(10, decimals)))
            .add(new bn_js_1.default(Math.round((Number(numberOrBn) % 1) * Math.pow(10, decimals))));
    }
}
exports.toBN = toBN;
function supplyAsNum(mint) {
    return amountAsNum(new bn_js_1.default(mint.supply.toString()), mint);
}
exports.supplyAsNum = supplyAsNum;
function numberWithCommas(x, decimals = 4) {
    return roundToDecimals(x, decimals).toLocaleString("en-US", {
        maximumFractionDigits: decimals,
    });
}
exports.numberWithCommas = numberWithCommas;
function roundToDecimals(num, decimals) {
    return Math.trunc(num * Math.pow(10, decimals)) / Math.pow(10, decimals);
}
exports.roundToDecimals = roundToDecimals;
function humanReadableBigint(bigint, decimals, round = decimals) {
    return numberWithCommas(roundToDecimals(toNumber(bigint, decimals), round));
}
exports.humanReadableBigint = humanReadableBigint;
function humanReadable(bn, decimalsOrMint) {
    return numberWithCommas(roundToDecimals(toNumber(bn, decimalsOrMint), typeof decimalsOrMint == "number"
        ? decimalsOrMint
        : decimalsOrMint.decimals));
}
exports.humanReadable = humanReadable;
const getSolanaKeypair = (secretKey) => web3_js_1.Keypair.fromSecretKey(Buffer.from(secretKey, "base64"));
exports.getSolanaKeypair = getSolanaKeypair;
const airdrop = (connection, to, amt) => __awaiter(void 0, void 0, void 0, function* () {
    const txSig = yield connection.requestAirdrop(to, amt * web3_js_1.LAMPORTS_PER_SOL);
    while (true) {
        const tx = yield connection.getTransaction(txSig, {
            commitment: 'confirmed'
        });
        if (tx) {
            break;
        }
        yield new Promise(resolve => setTimeout(resolve, 100));
        console.log(`waiting on airdrop`);
    }
});
exports.airdrop = airdrop;
function* packInstructions(as, prioFee = true) {
    let tx = new web3_js_1.Transaction({
        recentBlockhash: web3_js_1.PublicKey.default.toString(),
        feePayer: web3_js_1.PublicKey.default
    });
    if (prioFee) {
        tx.add(web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
            microLamports: Math.trunc(60000),
        }));
    }
    const txAs = [];
    for (const a of as) {
        tx.add(a.ix);
        txAs.push(a);
        try {
            const bl = [...tx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false
                })].length;
            if (bl > 1232) {
                console.log(`transaction size limit reached: ${bl}. aborting`);
                yield as.slice(0, txAs.length - 1);
                tx = new web3_js_1.Transaction({ recentBlockhash: web3_js_1.PublicKey.default.toString(), feePayer: web3_js_1.PublicKey.default });
            }
        }
        catch (e) {
            yield as.slice(0, txAs.length - 1);
            tx = new web3_js_1.Transaction({ recentBlockhash: web3_js_1.PublicKey.default.toString(), feePayer: web3_js_1.PublicKey.default });
        }
    }
    yield txAs;
}
exports.packInstructions = packInstructions;
const pingMillis = (connection, signer) => __awaiter(void 0, void 0, void 0, function* () {
    const now = Date.now();
    const tx = new web3_js_1.Transaction({
        feePayer: signer.publicKey,
        recentBlockhash: (yield connection.getLatestBlockhash()).blockhash,
    }).add(new web3_js_1.TransactionInstruction({
        keys: [{ pubkey: signer.publicKey, isSigner: true, isWritable: true }],
        data: Buffer.from("ping", "utf-8"),
        programId: new web3_js_1.PublicKey("MemoSq4gqABAXKb96qnH8TysNcWxMyWCqXgDLGmfcHr"),
    }));
    tx.sign(signer);
    yield (0, transaction_1.sendAndConfirmWithRetry)(connection, tx.serialize(), { skipPreflight: true }, 'confirmed', 300000);
    return Date.now() - now;
});
exports.pingMillis = pingMillis;
//# sourceMappingURL=utils.js.map